@@ -1,7 +1,8 @@
 /* infomap.c -- keymaps for Info.
-   $Id: infomap.c,v 1.28 2002/02/08 23:02:53 karl Exp $
+   $Id: infomap.c,v 1.2 2002/09/28 20:46:16 karl Exp $
 
-   Copyright (C) 1993, 97, 98, 99, 2001, 02 Free Software Foundation, Inc.
+   Copyright (C) 1993, 1997, 1998, 1999, 2001, 2002 Free Software
+   Foundation, Inc.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -505,8 +506,8 @@ initialize_vi_like_keymaps ()
   map['b'].function = ea_backward_word;
   map['d'].function = ea_kill_word;
   map['f'].function = ea_forward_word;
-  map['h'].function = ea_forward;
-  map['l'].function = ea_backward;
+  map['h'].function = ea_backward;
+  map['l'].function = ea_forward;
   map['w'].function = ea_forward_word;
   map['x'].function = ea_delete;
   map['X'].function = ea_kill_word;
