@@ -1,7 +1,7 @@
 /* cmds.c -- Texinfo commands.
-   $Id: cmds.c,v 1.69 2002/02/09 00:54:51 karl Exp $
+   $Id: cmds.c,v 1.14 2002/11/11 12:37:34 feloy Exp $
 
-   Copyright (C) 1998, 99, 2000, 01 Free Software Foundation, Inc.
+   Copyright (C) 1998, 1999, 2000, 2001, 2002 Free Software Foundation, Inc.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -22,6 +22,7 @@
 #include "defun.h"
 #include "files.h"
 #include "footnote.h"
+#include "html.h"
 #include "insertion.h"
 #include "lang.h"
 #include "macro.h"
@@ -47,7 +48,8 @@ void
   cm_image (), cm_kbd (), cm_key (), cm_no_op (), 
   cm_novalidate (), cm_not_fixed_width (), cm_r (),
   cm_strong (), cm_var (), cm_sc (), cm_w (), cm_email (), cm_url (),
-  cm_verb (), cm_documentdescription ();
+  cm_verb (), cm_copying (), cm_insert_copying (),
+  cm_documentdescription ();
 
 void
   cm_anchor (), cm_node (), cm_menu (), cm_xref (), cm_ftable (),
@@ -63,8 +65,8 @@ void
   cm_defcodeindex (), cm_result (), cm_expansion (), cm_equiv (),
   cm_print (), cm_error (), cm_point (), cm_today (), cm_flushleft (),
   cm_flushright (), cm_finalout (), cm_cartouche (), cm_detailmenu (),
-  cm_multitable (), cm_settitle (), cm_titlefont (), cm_tt (),
-  cm_verbatim (), cm_verbatiminclude (), cm_titlepage ();
+  cm_multitable (), cm_settitle (), cm_titlefont (), cm_titlepage (), cm_tie (), cm_tt (),
+  cm_verbatim (), cm_verbatiminclude ();
 
 /* Conditionals. */
 void cm_set (), cm_clear (), cm_ifset (), cm_ifclear ();
@@ -95,6 +97,7 @@ COMMAND command_table[] = {
   { "=", cm_accent, MAYBE_BRACE_ARGS },
   { "?", insert_self, NO_BRACE_ARGS },
   { "@", insert_self, NO_BRACE_ARGS },
+  { "\\", insert_self, NO_BRACE_ARGS },
   { "^", cm_accent_hat, MAYBE_BRACE_ARGS },
   { "`", cm_accent_grave, MAYBE_BRACE_ARGS },
   { "{", insert_self, NO_BRACE_ARGS },
@@ -139,6 +142,7 @@ COMMAND command_table[] = {
   { "command", cm_code, BRACE_ARGS },
   { "comment", cm_ignore_line, NO_BRACE_ARGS },
   { "contents", cm_contents, NO_BRACE_ARGS },
+  { "copying", cm_copying, NO_BRACE_ARGS },
   { "copyright", cm_copyright, BRACE_ARGS },
   { "ctrl", cm_obsolete, BRACE_ARGS },
   { "defcodeindex", cm_defcodeindex, NO_BRACE_ARGS },
@@ -202,6 +206,10 @@ COMMAND command_table[] = {
   { "env", cm_code, BRACE_ARGS },
   { "equiv", cm_equiv, BRACE_ARGS },
   { "error", cm_error, BRACE_ARGS },
+  { "evenfooting", cm_ignore_line, NO_BRACE_ARGS },
+  { "evenheading", cm_ignore_line, NO_BRACE_ARGS },
+  { "everyfooting", cm_ignore_line, NO_BRACE_ARGS },
+  { "everyheading", cm_ignore_line, NO_BRACE_ARGS },
   { "example", cm_example, NO_BRACE_ARGS },
   { "exampleindent", cm_exampleindent, NO_BRACE_ARGS },
   { "exclamdown", cm_special_char, BRACE_ARGS },
@@ -228,13 +236,18 @@ COMMAND command_table[] = {
   { "ifinfo", cm_ifinfo, NO_BRACE_ARGS },
   { "ifnothtml", cm_ifnothtml, NO_BRACE_ARGS },
   { "ifnotinfo", cm_ifnotinfo, NO_BRACE_ARGS },
+  { "ifnotplaintext", cm_ifnotplaintext, NO_BRACE_ARGS },
   { "ifnottex", cm_ifnottex, NO_BRACE_ARGS },
+  { "ifnotxml", cm_ifnotxml, NO_BRACE_ARGS },
+  { "ifplaintext", cm_ifplaintext, NO_BRACE_ARGS },
   { "ifset", cm_ifset, NO_BRACE_ARGS },
   { "iftex", cm_iftex, NO_BRACE_ARGS },
+  { "ifxml", cm_ifxml, NO_BRACE_ARGS },
   { "ignore", command_name_condition, NO_BRACE_ARGS },
   { "image", cm_image, BRACE_ARGS },
   { "include", cm_include, NO_BRACE_ARGS },
   { "inforef", cm_inforef, BRACE_ARGS },
+  { "insertcopying", cm_insert_copying, NO_BRACE_ARGS },
   { "item", cm_item, NO_BRACE_ARGS },
   { "itemize", cm_itemize, NO_BRACE_ARGS },
   { "itemx", cm_itemx, NO_BRACE_ARGS },
@@ -257,6 +270,8 @@ COMMAND command_table[] = {
   { "noindent", cm_novalidate, NO_BRACE_ARGS },
   { "nwnode", cm_node, NO_BRACE_ARGS },
   { "o", cm_special_char, BRACE_ARGS },
+  { "oddfooting", cm_ignore_line, NO_BRACE_ARGS },
+  { "oddheading", cm_ignore_line, NO_BRACE_ARGS },
   { "oe", cm_special_char, BRACE_ARGS },
   { "option", cm_code, BRACE_ARGS },
   { "page", cm_no_op, NO_BRACE_ARGS },
@@ -308,10 +323,11 @@ COMMAND command_table[] = {
   { "tab", cm_tab, NO_BRACE_ARGS },
   { "table", cm_table, NO_BRACE_ARGS },
   { "tex", cm_tex, NO_BRACE_ARGS },
+  { "tie", cm_tie, BRACE_ARGS },
   { "tieaccent", cm_accent, MAYBE_BRACE_ARGS },
   { "tindex", cm_tindex, NO_BRACE_ARGS },
   { "titlefont", cm_titlefont, BRACE_ARGS },
-  { "titlepage", command_name_condition, NO_BRACE_ARGS },
+  { "titlepage", cm_titlepage, NO_BRACE_ARGS },
   { "today", cm_today, BRACE_ARGS },
   { "top", cm_top, NO_BRACE_ARGS  },
   { "u", cm_accent, MAYBE_BRACE_ARGS },
@@ -333,6 +349,7 @@ COMMAND command_table[] = {
   { "vindex", cm_vindex, NO_BRACE_ARGS },
   { "vtable", cm_vtable, NO_BRACE_ARGS },
   { "w", cm_w, BRACE_ARGS },
+  { "xml", cm_html, NO_BRACE_ARGS },
   { "xref", cm_xref, BRACE_ARGS },
 
   /* Deprecated commands.  These used to be for italics.  */
@@ -375,9 +392,9 @@ insert_space (arg)
   if (arg == START)
     {
       if (xml && !docbook)
-	xml_insert_entity ("space");
+        xml_insert_entity ("space");
       else
-	add_char (' ');
+        add_char (' ');
     }
 }
 
@@ -406,11 +423,12 @@ cm_dots (arg)
   if (arg == START)
     {
       if (xml && !docbook)
-	xml_insert_entity ("dots");
+        xml_insert_entity ("dots");
       else if (docbook)
-	xml_insert_entity ("hellip");
+        xml_insert_entity ("hellip");
       else
-	add_word (html ? "<small>...</small>" : "...");
+        add_word (html && !in_fixed_width_font
+                  ? "<small class=\"dots\">...</small>" : "...");
     }
 }
 
@@ -429,7 +447,8 @@ cm_enddots (arg)
 	  add_char ('.');
 	}
       else
-	add_word (html ? "<small>...</small>." : "....");
+	add_word (html && !in_fixed_width_font 
+                  ? "<small class=\"enddots\">....</small>" : "....");
     }
 }
 
@@ -541,24 +560,24 @@ cm_code (arg)
     xml_insert_element (CODE, arg);
   else
     {
-  extern int printing_index;
+      extern int printing_index;
 
-  if (arg == START)
-    {
-      in_fixed_width_font++;
+      if (arg == START)
+        {
+          in_fixed_width_font++;
 
-      if (html)
+          if (html)
+            insert_html_tag (arg, "code");
+          else if (!printing_index)
+            add_char ('`');
+        }
+      else if (html)
         insert_html_tag (arg, "code");
-      else if (!printing_index)
-        add_char ('`');
-    }
-  else if (html)
-    insert_html_tag (arg, "code");
-  else
-    {
-      if (!printing_index)
-        add_meta_char ('\'');
-    }
+      else
+        {
+          if (!printing_index)
+            add_meta_char ('\'');
+        }
     }
 }
 
@@ -592,9 +611,10 @@ cm_url (arg, start, end)
   else if (html)
     {
       if (arg == START)
-        add_word ("&lt;<code>");
-      else
-	add_word ("</code>&gt;");
+        add_word ("&lt;");
+      insert_html_tag (arg, "code");
+      if (arg != START)
+        add_word ("&gt;");
     }
   else
     if (arg == START)
@@ -860,25 +880,9 @@ cm_r (arg)
     xml_insert_element (R, arg);
   else
     {
-      extern int printing_index;
+      if (html)
+	insert_html_tag (arg, "");
 
-      /* People use @r{} in index entries like this:
-	 
-      @findex foo@r{, some text}
-      
-      This is supposed to produce output as if the entry were saying
-      "@code{foo}, some text", since the "fn" index is typeset as
-      @code.  The following attempts to do the same in HTML.  Note that
-      this relies on the fact that only @code bumps up the variable
-      in_fixed_width_font while processing index entries in HTML mode.  */
-      if (html && printing_index)
-	{
-	  int level = in_fixed_width_font;
-	  
-	  while (level--)
-	    insert_html_tag (arg == START ? END : START, "code");
-	}
-      
       not_fixed_width (arg);
     }
 }
@@ -890,7 +894,27 @@ cm_titlefont (arg)
   if (xml)
     xml_insert_element (TITLEFONT, arg);
   else
-  not_fixed_width (arg);
+   {
+     not_fixed_width (arg);
+     if (html)
+	{
+	  html_title_written = 1; /* suppress title from @settitle */
+	  if (arg == START)
+	    add_word ("<h1 class=\"titlefont\">");
+	  else
+	    add_word ("</h1>\n");
+	}
+   }
+}
+
+int titlepage_cmd_present = 0;
+
+void
+cm_titlepage (arg)
+     int arg;
+{
+  titlepage_cmd_present = 1;
+  command_name_condition ();
 }
 
 /* Various commands are no-op's. */
@@ -920,6 +944,22 @@ cm_w (arg, start, end)
 }
 
 
+/* An unbreakable word space.  Same as @w{ } for makeinfo, but different
+   for TeX (the space stretches and stretches, and does not inhibit
+   hyphenation).  */
+void
+cm_tie (arg)
+    int arg;
+{
+  if (arg == START)
+    {
+      cm_w (START);
+      add_char (' ');
+    }
+  else
+    cm_w (END);
+}
+
 /* Explain that this command is obsolete, thus the user shouldn't
    do anything with it. */
 static void
@@ -962,11 +1002,18 @@ cm_settitle ()
 {
   if (xml)
     {
-      xml_begin_document ();
+      xml_begin_document (current_output_filename);
       xml_insert_element (SETTITLE, START);
+      xml_in_book_title = 1;
       get_rest_of_line (0, &title);
       execute_string ("%s", title);
+      xml_in_book_title = 0;
       xml_insert_element (SETTITLE, END);
+      if (docbook && !xml_in_bookinfo)
+	{
+	  xml_insert_element (BOOKINFO, START);
+	  xml_in_bookinfo = 1;
+	}
     }
   else
     get_rest_of_line (0, &title);
@@ -1016,7 +1063,8 @@ cm_sp ()
       int save_filling_enabled = filling_enabled;
       filling_enabled = 0;
       
-      close_paragraph ();
+      /* close_paragraph generates an extra blank line.  */
+      close_single_paragraph ();
 
       if (lines && html && !executing_string)
 	html_output_head ();
@@ -1102,7 +1150,7 @@ cm_center ()
   execute_string ("%s", (char *)line);
   free (line);
   uninhibit_output_flushing ();
-   if (html)
+  if (html)
     add_word ("</div>");
 
    else
@@ -1201,7 +1249,7 @@ cm_exdent ()
   int save_indent = current_indent;
   int save_in_fixed_width_font = in_fixed_width_font;
 
-  /* Read argument  */
+  /* Read argument.  */
   get_rest_of_line (0, &line);
 
   /* Exdent the output.  Actually this may be a no-op.   */
@@ -1229,6 +1277,7 @@ cm_exdent ()
 
   current_indent = save_indent;
   in_fixed_width_font = save_in_fixed_width_font;
+  start_paragraph ();
 }
 
 /* 
