@@ -1,7 +1,7 @@
 /* node.c -- nodes for Texinfo.
-   $Id: node.c,v 1.31 2002/02/23 19:12:15 karl Exp $
+   $Id: node.c,v 1.5 2002/11/05 03:04:26 karl Exp $
 
-   Copyright (C) 1998, 99, 2000, 01, 02 Free Software Foundation, Inc.
+   Copyright (C) 1998, 1999, 2000, 2001, 2002 Free Software Foundation, Inc.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -306,7 +306,12 @@ remember_node (node, prev, next, up, position, line_no, fname, flags)
         node_number++;
         new->number = node_number;
       }
-    new->html_fname = fname;
+    if (fname)
+      new->html_fname = fname;
+    else
+      /* This happens for Top node under split-HTML, for example.  */
+      new->html_fname
+	= normalize_filename (filename_part (current_output_filename));
     new->next_ent = tag_table;
     tag_table = new;
   }
@@ -599,7 +604,7 @@ cm_node ()
   no_indent = 1;
   if (xml)
     {
-      xml_begin_document ();
+      xml_begin_document (current_output_filename);
       xml_begin_node ();
       if (!docbook)
 	{
@@ -914,8 +919,9 @@ cm_node ()
 	}
 
       if (splitting || !no_headers)
-        { /* Navigation bar.   The <p> avoids the links area running
-             on with old Lynxen.  */
+        { /* Navigation bar. */
+          add_word ("<div class=\"node\">\n");
+          /* The <p> avoids the links area running on with old Lynxen. */
           add_word_args ("<p>%s\n", splitting ? "" : "<hr>");
           add_word_args ("%s<a name=\"", _("Node:"));
           tem = expand_node_name (node);
@@ -928,7 +934,7 @@ cm_node ()
               tem = expansion (next, 0);
 	      add_word (",\n");
 	      add_word (_("Next:"));
-	      add_word ("<a rel=next href=\"");
+	      add_word ("<a rel=\"next\" accesskey=\"n\" href=\"");
 	      add_anchor_name (tem, 1);
 	      add_word_args ("\">%s</a>", tem);
               free (tem);
@@ -938,7 +944,7 @@ cm_node ()
               tem = expansion (prev, 0);
 	      add_word (",\n");
 	      add_word (_("Previous:"));
-	      add_word ("<a rel=previous href=\"");
+	      add_word ("<a rel=\"previous\" accesskey=\"p\" href=\"");
 	      add_anchor_name (tem, 1);
 	      add_word_args ("\">%s</a>", tem);
               free (tem);
@@ -948,7 +954,7 @@ cm_node ()
               tem = expansion (up, 0);
 	      add_word (",\n");
 	      add_word (_("Up:"));
-	      add_word ("<a rel=up href=\"");
+	      add_word ("<a rel=\"up\" accesskey=\"u\" href=\"");
 	      add_anchor_name (tem, 1);
 	      add_word_args ("\">%s</a>", tem);
               free (tem);
@@ -956,6 +962,7 @@ cm_node ()
           /* html fixxme: we want a `top' or `contents' link here.  */
 
           add_word_args ("\n%s<br>\n", splitting ? "<hr>" : "");
+      	  add_word ("</div>\n");
         }
     }
   else if (docbook)
@@ -1149,7 +1156,8 @@ cm_anchor (arg)
       xml_insert_element (ANCHOR, END);
     }
   /* Save it in the tag table.  */
-  remember_node (anchor, NULL, NULL, NULL, output_position + output_column,
+  remember_node (anchor, NULL, NULL, NULL,
+                 output_position + output_paragraph_offset,
                  line_number, fname_for_anchor, TAG_FLAG_ANCHOR);
 }
 
